// EnumSQLServer.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
#include <set>

#include <atldbcli.h>


//////////////////////////////////////////////////////////////////////////
// Main function to enumerate all servers with the appropriate
// known enumerators.

typedef std::set<CString>  TSET_CString;

void EnumSQLServer(TSET_CString &setSQLServer)
{
	// We may need the local server name. 
	// We replace the token (local) with the current computer name.
	CString strCompLocal;
	DWORD dwLen = MAX_COMPUTERNAME_LENGTH+1;
	::GetComputerName(CStrBuf(strCompLocal,dwLen),&dwLen);

	// Loop over all enumerators we know
	static const PCWSTR aEnumerator[] =
	{
		L"SQLNCLI10 Enumerator",		// SQL 2008
		L"SQLNCLI Enumerator",			// SQL 2005
		L"SQLOLEDB Enumerator"			// SQL 2000
	};

	// Try all enumerators
	for (int i=0; i<_countof(aEnumerator); ++i)
	{
		// Check if we have an enumerator
		bool bFoundAny = false;
		HRESULT hr;
		CLSID clsid;
		hr = CLSIDFromProgID(aEnumerator[i],&clsid);
		if (SUCCEEDED(hr))
		{
			// Open enumerator and loop over all entries
			CEnumerator enumrator;
			hr = enumrator.Open(&clsid);
			if (SUCCEEDED(hr))
			{
				while ((hr=enumrator.MoveNext())==S_OK)
				{
					CString strServerName(enumrator.m_szName);

					// Skip empty server names 
					// (older enumerators return sometimes an empty name)
					if (strServerName.IsEmpty())
						continue;

					// Some enumerators return (local) for a local main
					// SQL server instance
					if (strServerName.CompareNoCase(_T("(local)"))==0)
					{
						ATLTRACE(__FUNCTION__ " found local computer\n");
						strServerName = strCompLocal;
					}

					// get uppercase server name
					strServerName.MakeUpper();

					// Insert in list and avoid duplicates with this, if
					// developer decides not to break the loop after the first
					// enumerator.
					if (setSQLServer.insert(strServerName).second)
						ATLTRACE(__FUNCTION__ " found server %s\n",CT2A(strServerName.GetString()));
					bFoundAny = true;
				}
			}

			// After we have found data in one enumerator. There is no need to do this again.
			// But a developer might decide to do this for every enumerator
			if (bFoundAny)
				break;
		}
	}
}

int _tmain(int /*argc*/, _TCHAR* /*argv*/[])
{
	if (FAILED(CoInitialize(NULL)))
		return 2;

	TSET_CString setSQLServer;
	EnumSQLServer(setSQLServer);

	for (TSET_CString::iterator it=setSQLServer.begin(); it!=setSQLServer.end(); ++it)
		std::cout << CT2A(*it) << std::endl;

	CoUninitialize();
	return 0;
}

//////////////////////////////////////////////////////////////////////////
// This is just old legacy code that uses SQL DMO to enumerate the 
// servers. 
// Article: 
//		http://support.microsoft.com/kb/287737/en-us
//		How to enumerate available instances of SQL Server by 
//		using the SQLDMO components

#ifdef DMO_LEGACY_CODE
void EnumSQLServerWithDMO(TSET_CString &setSQLServer)
{
	// We may need the local server name. 
	// We replace the token (local) with the current computer name.
	CString strCompLocal;
	DWORD dwLen = MAX_COMPUTERNAME_LENGTH+1;
	::GetComputerName(CStrBuf(strCompLocal,dwLen),&dwLen);


	// 1. We SQL try DMO
	CComPtr<ISQLDMOApplication> spISQLDMOApplication;
	HRESULT hr;
	if (SUCCEEDED(hr = CoCreateInstance(CLSID_SQLDMOApplication, NULL, CLSCTX_INPROC_SERVER,
										IID_ISQLDMOApplication, (LPVOID*)&spISQLDMOApplication)))
	{
		CComPtr<ISQLDMONameList> spISQLDMONameList;
		if (SUCCEEDED(hr = spISQLDMOApplication->ListAvailableSQLServers(&spISQLDMONameList)))
		{
			LONG lServersCount= 0;
			if (SUCCEEDED(hr = spISQLDMONameList->GetCount(&lServersCount)))
			{
				for (LONG lPos= 0; lPos < lServersCount; ++lPos) 
				{
					CComBSTR bstrServerName;
					if FAILED(hr = spISQLDMONameList->GetItemByOrd(lPos,&bstrServerName)) 
					{
						ATLTRACE("ISQLDMONameList::GetCount() Failed\n");
						return;
					}

					// get uppercase server name
					CString strServerName(bstrServerName);
					strServerName.MakeUpper();

					if (strServerName.CompareNoCase(_T("(local)"))==0)
					{
						ATLTRACE(__FUNCTION__ " found local computer\n");
						strServerName = strCompLocal;
					}

					// Insert into set
					if (setSQLServer.insert(strServerName).second)
						ATLTRACE(__FUNCTION__ " found server %s\n",CT2A(strServerName.GetString()));
				}
			}
			else
			{
				ATLTRACE("ISQLDMONameList::GetCount() Failed\n");
			}
		}
		else
		{
			ATLTRACE("ISQLDMOApplication::ListAvailableSQLServers() Failed\n");
		}
	}
	else
	{
		ATLTRACE("CoCreateInstance(CLSID_SQLDMOApplication) Failed\n");
	}
}

#endif